@@ -1,6 +1,6 @@
 #!/bin/sh
 #set -xv
-UPGRADE_VER="1.0.12"
+UPGRADE_VER="1.0.14"
 TARGET_PREFIX=sp80_upgrade_
 BIN_PATH=/home/root
 CARD_PATH=${MOUNTR}
@@ -52,6 +52,12 @@ displ() {
  echo "Upgrading ${1}" | tee -a ${LOG}
 }
 
+fs_flush() {
+ echo "fs_flush" | tee -a ${LOG}
+ free | tee -a ${LOG}
+ echo 3 > /proc/sys/vm/drop_caches
+}
+
 do_reboot() {
   if [ -f ${CARD_PATH}/upl_start.log ]; then
     rm -rf ${CARD_PATH}/${RFS_NAME}* ${CARD_PATH}/${KERN_NAME}* ${CARD_PATH}/${BOOT_NAME}* ${CARD_PATH}/upl_start.log
@@ -70,6 +76,7 @@ do_sys_check() {
   dd_len=$(($chk_len/512+1))
   chk_hash="n/a"
   #chk_hash=`dd if=$OUT_DD_DEV skip=$2 count=$dd_len | head -c $chk_len | md5sum | awk '{split($0,x," ");printf("%s",x[1])}'`
+  fs_flush
   $NAND_WR -d ${OUT_DD_DEV} -o $2 -r ${chk_name}.out ${chk_len} | tee -a ${LOG}
   if [ $? -eq 0 ]; then
    chk_hash=`cat ${chk_name}.out | md5sum | awk '{split($0,x," ");printf("%s",x[1])}'`
@@ -125,6 +132,7 @@ if [ -f ${1} ]; then
  echo "Check $1 at " $(date) | tee -a ${LOG}
  hash1=$(grep ${1} gnss.md5 | awk '{split($0,x," ");printf("%s",x[1])}')
  echo "Orig hash $hash1" | tee -a ${LOG}
+ fs_flush
  hash2=$(md5sum ${1} | awk '{split($0,x," ");printf("%s",x[1])}')
  echo "File hash $hash2" | tee -a ${LOG}
  if [ "${hash1}" != "${hash2}" ]; then
@@ -163,6 +171,10 @@ if [ -e "/dev/mtd10" ]; then
  OUT_CHAR_DEV=/dev/mtd6
 fi
 
+swapoff -a
+#while true; do echo 3 > /proc/sys/vm/drop_caches ; sleep 1 ; done &
+#echo "drop_caches started"
+
 fl_continue=0
 if [ "$smod" -eq "4" ]; then fl_continue=1 ; fi
 if [ "$smod" -eq "14" ]; then fl_continue=1 ; fi
@@ -196,6 +208,11 @@ if [ "$fl_continue" -eq "0" ]; then
 
  ##### SYSTEM upgrade ###########################
  fl_kupl=0
+ ps | tee -a ${LOG}
+ if [ "$smod" -eq "4" ]; then
+   echo "SD-Card $(busybox hdparm /dev/mmcblk0)" | tee -a ${LOG}
+ fi
+ echo "/proc/sys/vm/drop_caches=$(cat /proc/sys/vm/drop_caches)"
  flist="`tar -t$TAR_OPT $TAR_FILE`"
  echo "flist:$flist" | tee -a ${LOG}
  # tar -x$TAR_OPT $TAR_FILE $BOOT_NAME $XLDR_NAME
@@ -203,6 +220,7 @@ if [ "$fl_continue" -eq "0" ]; then
  fl_kern=`echo $flist | grep -c "$KERN_TAR"`
  fl_boot=`echo $flist | grep -c "$BOOT_TAR"`
  if [[ "${fl_rfs}" -ne "0" && "${fl_kern}" -ne "0" && "${fl_boot}" -ne "0" ]]; then
+  fs_flush
   tar -x$TAR_OPT $TAR_FILE $NWRITE_NAME
   if [ ! -f $NWRITE_NAME ]; then
    echo "Can not find $NWRITE_NAME. Upgrade cancelled" 
@@ -211,6 +229,7 @@ if [ "$fl_continue" -eq "0" ]; then
    echo "UERR 1" > /proc/sp4
    do_reboot 1 "Error 1.1"
   fi
+  fs_flush
   tar -xO$TAR_OPT $TAR_FILE $BOOT_TAR | tar -xj
   ls -l | tee -a ${LOG}
   if [ ! -f $NAND_WR ]; then
@@ -230,15 +249,18 @@ if [ "$fl_continue" -eq "0" ]; then
   displ "Step 1/9"
   echo "unpack $RFS_NAME to ${CARD_PATH}" | tee -a ${LOG}
   cd ${CARD_PATH}
+  fs_flush
   tar -xO$TAR_OPT $TAR_FILE $RFS_TAR | tar -xj
   ret=$?
   echo "Unpack ret $ret" | tee -a ${LOG}
   ls -l ${RFS_NAME}* | tee -a ${LOG}
   if [ -f ${RFS_NAME}.md5 ]; then
+      fs_flush
       hash1=$(cat ${RFS_NAME}.md5)
       echo "Orig hash $hash1" | tee -a ${LOG}
   fi
   if [ $ret -eq 0 ]; then
+      fs_flush
       hash2=$(md5sum ${RFS_NAME})
       echo "File hash $hash2" | tee -a ${LOG}
       if [ "${hash1}" != "${hash2}" ]; then
@@ -258,6 +280,7 @@ if [ "$fl_continue" -eq "0" ]; then
   displ "Step 2/9"
   echo "unpack $KERN_NAME to ${CARD_PATH}" | tee -a ${LOG}
   cd ${CARD_PATH}
+  fs_flush
   tar -xO$TAR_OPT $TAR_FILE $KERN_TAR | tar -xj
   ret=$?
   echo "Unpack ret $ret" | tee -a ${LOG}
@@ -268,6 +291,7 @@ if [ "$fl_continue" -eq "0" ]; then
       echo "Orig hash $hash1" | tee -a ${LOG}
   fi
   if [ $ret -eq 0 ]; then
+      fs_flush
       hash2=$(md5sum ${KERN_NAME})
       echo "File hash $hash2" | tee -a ${LOG}
       if [ "${hash1}" != "${hash2}" ]; then
@@ -295,6 +319,7 @@ if [ "$fl_continue" -eq "0" ]; then
       echo "Orig hash $hash1" | tee -a ${LOG}
   fi
   if [ $ret -eq 0 ]; then
+      fs_flush
       hash2=$(md5sum ${BOOT_NAME})
       echo "File hash $hash2" | tee -a ${LOG}
       if [ "${hash1}" != "${hash2}" ]; then
@@ -309,6 +334,7 @@ if [ "$fl_continue" -eq "0" ]; then
  if [[ "${fl_rfs}" -ne "0" && "${fl_kern}" -ne "0" && "${fl_boot}" -ne "0" ]]; then
       displ "Step 3/9"
       echo "wr $RFS_NAME to $OUT_DD_DEV" | tee -a ${LOG}
+      fs_flush
       $NAND_WR -d ${OUT_DD_DEV} -o 0x1000000 -i ${CARD_PATH}/${RFS_NAME} | tee -a ${LOG}
       if [ ! $? -eq 0 ]; then
           echo "$RFS_NAME record failure" | tee -a ${LOG}
@@ -318,6 +344,7 @@ if [ "$fl_continue" -eq "0" ]; then
       fi
       displ "Step 4/9"
       echo "wr $KERN_NAME to $OUT_DD_DEV" | tee -a ${LOG}
+      fs_flush
       $NAND_WR -d ${OUT_DD_DEV} -o 0x800000 -i ${CARD_PATH}/${KERN_NAME} | tee -a ${LOG}
       if [ ! $? -eq 0 ]; then
           echo "$KERN_NAME record failure" | tee -a ${LOG}
@@ -329,6 +356,7 @@ if [ "$fl_continue" -eq "0" ]; then
       echo "record BOOT" | tee -a ${LOG}
       if [ -f ${CARD_PATH}/$BOOT_NAME ]; then
         echo "wr $BOOT_NAME to $OUT_DD_DEV" | tee -a ${LOG}
+        fs_flush
         $NAND_WR -d ${OUT_DD_DEV} -o 0x400000 -i ${CARD_PATH}/${BOOT_NAME} | tee -a ${LOG}
         if [ $? -ne 0 ]; then
             echo "$BOOT_NAME record failure" | tee -a ${LOG}
@@ -378,6 +406,7 @@ fi
 if [ -f ${CARD_PATH}/x-load-upd.log ]; then
  rm -f ${CARD_PATH}/x-load-upd.log
  echo "Check $XLDR_NAME u-boot format" | tee -a ${LOG}
+ fs_flush
  tar -x$TAR_OPT $TAR_FILE $NWRITE_NAME
  new_pattern="12345678.f1000001"
  new_flag=`dd if=${BOOT_DD_DEV} count=1 skip=2048 | head -c 68 | tail -c 8 | hexdump | head -n 1 | awk '{split($0,x," ");printf("%s%s.%s%s",x[3],x[2],x[5],x[4])}'`
@@ -388,6 +417,7 @@ if [ -f ${CARD_PATH}/x-load-upd.log ]; then
    rm -f ${NWRITE_NAME}
  fi
  echo "Check $XLDR_NAME image" | tee -a ${LOG}
+ fs_flush
  tar -xO$TAR_OPT $TAR_FILE $BOOT_TAR | tar -xj
  ret=$?
  echo "Unpack ret $ret" | tee -a ${LOG}
@@ -420,6 +450,7 @@ if [ -f ${CARD_PATH}/x-load-upd.log ]; then
  if [ -s ${NWRITE_NAME} ]; then
      if [ -f $XLDR_NAME ]; then
        echo "wr $XLDR_NAME to $OUT_DD_DEV" | tee -a ${LOG}
+       fs_flush
        $NAND_WR -d ${OUT_DD_DEV} -o 0x0 -i ${XLDR_NAME} | tee -a ${LOG}
        if [ $? -ne 0 ]; then
            echo "$XLDR_NAME record failure" | tee -a ${LOG}
@@ -448,9 +479,23 @@ mount | tee -a ${LOG}
 #flist2=`echo ${flist} | sed 's/'${RFS_TAR}'/ /;s/'${KERN_TAR}'/ /'`
 #echo "untar $flist2"
 echo "untar ${PMU_TAR} from ${TAR_FILE}" | tee -a ${LOG}
+fs_flush
 tar -xO${TAR_OPT} ${TAR_FILE} ${PMU_TAR} | tar -xj
+if [ $? -ne 0 ]; then
+		echo "Tar unpack error. Exiting" | tee -a  ${LOG}
+		echo "Ending upgrade script at " $(date) | tee -a  ${LOG}
+        echo "UERR 1" > /proc/sp4
+        do_reboot 1 "Error 6.1"
+fi
 echo "untar ${GNSS_TAR} from ${TAR_FILE}" | tee -a ${LOG}
+fs_flush
 tar -xO${TAR_OPT} ${TAR_FILE} ${GNSS_TAR} | tar -xj
+if [ $? -ne 0 ]; then
+		echo "Tar unpack error. Exiting" | tee -a  ${LOG}
+		echo "Ending upgrade script at " $(date) | tee -a  ${LOG}
+        echo "UERR 1" > /proc/sp4
+        do_reboot 1 "Error 6.2"
+fi
 #tar -xO${TAR_OPT} ${TAR_FILE} ${APPL_TAR} | tar -xj
 
 #copy_app ${SP4_GNSS_SH}
@@ -469,13 +514,19 @@ CRC32_EXE=./${CRC32_NAME}
 
 CPU1_BOOT_UPGRADE_FILE=./${CPU1_BOOT_NAME}
 echo "untar ${CRC32_NAME} from ${TAR_FILE}" | tee -a ${LOG}
+fs_flush
 tar -x${TAR_OPT} ${TAR_FILE} ${CRC32_NAME}
-
+if [ $? -ne 0 ]; then
+		echo "Tar unpack error. Exiting" | tee -a  ${LOG}
+		echo "Ending upgrade script at " $(date) | tee -a  ${LOG}
+        echo "UERR 1" > /proc/sp4
+        do_reboot 1 "Error 6.3"
+fi
+ls -l * | tee -a ${LOG}
 if [ -f ${CPU1_BOOT_UPGRADE_FILE} ]; then
 	if [ -f ${SP4LOADER} ];	then
         displ "Step 7/9"
 		echo "Upgrading CPU1 ${CPU1_BOOT_NAME} at " $(date) | tee -a ${LOG}
-		ls -l * | tee -a ${LOG}
 		rm -rf ./g5.log
 
    check_gnss ${CPU1_BOOT_NAME}
@@ -550,10 +601,10 @@ fi
 
 G5_UPGRADE_FILE=./${G5_SENSOR_NAME}
 
-if [ -f ${G5_UPGRADE_FILE} ];
+if [ -f ${G5_UPGRADE_FILE} ]; 
 then
-	if [ -f ${SP4LOADER} ];
-	then
+	if [ -f ${SP4LOADER} ];	
+    then
         displ "Step 8/9"
         echo "Upgrading G5 at " $(date) | tee -a ${LOG}
 		echo "Upgrading G5 sensor" | tee -a ${LOG}
@@ -566,6 +617,7 @@ then
     	then
 			crc=`${CRC32_EXE} ${G5_UPGRADE_FILE}`
             echo "${G5_UPGRADE_FILE} crc ${crc}" | tee -a ${LOG}
+            fs_flush
 		    ${SP4LOADER} -f ${G5_UPGRADE_FILE} -p /dev/ttySP1 -c ${crc} 2>&1
         else
 		    ${SP4LOADER} -f ${G5_UPGRADE_FILE} -p /dev/ttySP1 > /dev/null 2>&1
@@ -575,6 +627,7 @@ then
             echo "=== cpu1_deb.log start ===" | tee -a ${LOG}
 	    	cat ./cpu1_deb.log >> ${LOG}
             echo "=== cpu1_deb.log stop ===" | tee -a ${LOG}
+            fs_flush
 		    ${SP4LOADER} -r -f ${G5_UPGRADE_FILE} -p /dev/ttySP1 > /dev/null 2>&1
         fi
 		if [ ! $? -eq 0 ]; then
@@ -609,7 +662,6 @@ then
 
         check_gnss ${CPU1_XLDR_NAME}
         check_gnss ${SP4_LOADER_NAME}
-
 		${SP4LOADER} -x -f ${CPU1_XLDR_UPGRADE_FILE} -p /dev/ttySP1 >> ./g5.log 2>&1
 		if [ ! $? -eq 0 ];
 		then
